#include "stdafx.h"
#include "Util_MString.h"
#include <actrans.h>
#include "AcToRtf.h"

/****************************************************************************
**  Acad::ErrorStatus getTextStyleName(CString& styleName,
**  AcDbObjectId styleId)
**  
**	  
**
**  **avb
**
****************************/


Acad::ErrorStatus 
getTextStyleName(CString& styleName, AcDbObjectId styleId)
{
	if(AcDbObjectId::kNull == styleId)
		return Acad::eNullObjectId;
	AcDbObject *pObj;
	acdbOpenAcDbObject(pObj, styleId, AcDb::kForRead);
	AcDbTextStyleTableRecord *pStyle;
	pStyle = AcDbTextStyleTableRecord::cast(pObj);
	if(pStyle == NULL)
	{
		pObj->close();
		return Acad::eWrongObjectType;
	}
	char * pStyleName;
	pStyle->getName(pStyleName);
	styleName = pStyleName;
	pStyle->close();
	return Acad::eOk;
}

/****************************************************************************
**
**  Acad::ErrorStatus getSysVar (LPCTSTR varName, CString& val)
**
**  **avb
**
*************************************/

Acad::ErrorStatus
getSysVar(LPCTSTR varName, CString& val)
{
    resbuf rb;
    if (acedGetVar(varName, &rb) == RTNORM) {
        ASSERT(rb.restype == RTSTR);
        val = rb.resval.rstring;
        free(rb.resval.rstring);
		return(Acad::eOk);
    }
    else
        return(Acad::eInvalidInput);

	
	
}
/****************************************************************************
**
**  Acad::ErrorStatus getSysVar (LPCTSTR varName, double& val)
**
**  **avb
**
*************************************/

Acad::ErrorStatus
getSysVar(LPCTSTR varName, double& val)
{
    resbuf rb;
    if (acedGetVar(varName, &rb) == RTNORM) {
        ASSERT(rb.restype == RTREAL);
        val = rb.resval.rreal;
		return(Acad::eOk);
    }
    else
        return(Acad::eInvalidInput);
}

/****************************************************************************
**
**  Acad::ErrorStatus getSysVar (LPCTSTR varName, int& val)
**
**  **avb
**
*************************************/

Acad::ErrorStatus
getSysVar(LPCTSTR varName, int& val)
{
    resbuf rb;
    if (acedGetVar(varName, &rb) == RTNORM) {
        ASSERT(rb.restype == RTREAL);
        val = rb.resval.rint;
        return(Acad::eOk);
    }
    else
        return(Acad::eInvalidInput);
}

/******************************************************************
**	AcDbObjectId getTextStyleId(const CString styleName)
**  
**	  Id  
**
**  **avb
**
****************************/

AcDbObjectId
getTextStyleId(const CString styleName)
{
	AcDbDatabase * pDb = NULL;
	pDb = acdbHostApplicationServices()->workingDatabase();
	/*if(pDb == NULL)
		acrx_abort("!%s@%d: %s", __FILE__, __LINE__, "No database!");*/
	AcDbTextStyleTable *pTsTable;
	pDb->getTextStyleTable(pTsTable, AcDb::kForRead);
	AcDbObjectId styleId;
	pTsTable->getAt(styleName, styleId);
	pTsTable->close();
	return styleId;
}



/******************************************************************
**  Acad::ErrorStatus
**	postToDatabase(AcDbEntity* pEnt,AcDbObjectId& idObj)
**  
**	   MODEL_SPACE   .
**
**  **avb
**
****************************/

Acad::ErrorStatus
postToDatabase(/*[in]*/AcDbEntity* pEnt,/*[out]*/AcDbObjectId& idObj)
{
	int val;
	getSysVar("TILEMODE", val);
	Acad::ErrorStatus	  es;
	AcDbBlockTable*		pBlockTable;
	AcDbBlockTableRecord*  pSpaceRecord;
    if (acdbHostApplicationServices()->workingDatabase()==NULL)
        return Acad::eNoDatabase;
	if (val==1)
	{
    if ((es = acdbHostApplicationServices()->workingDatabase()->getBlockTable(pBlockTable, AcDb::kForRead))==Acad::eOk){
        if ((es = pBlockTable->getAt(ACDB_MODEL_SPACE, pSpaceRecord, AcDb::kForWrite))==Acad::eOk){
            if ((es = pSpaceRecord->appendAcDbEntity(idObj, pEnt))==Acad::eOk)
                pEnt->close();
            pSpaceRecord->close();
			

        }
	    pBlockTable->close();
    }
	}
	else if (val==0)
	{
    if ((es = acdbHostApplicationServices()->workingDatabase()->getBlockTable(pBlockTable, AcDb::kForRead))==Acad::eOk){
        if ((es = pBlockTable->getAt(ACDB_PAPER_SPACE, pSpaceRecord, AcDb::kForWrite))==Acad::eOk){
            if ((es = pSpaceRecord->appendAcDbEntity(idObj, pEnt))==Acad::eOk)
                pEnt->close();
            pSpaceRecord->close();
        }
	    pBlockTable->close();
    }
	
	}
return es;
}

/******************************************************************
**	void refresh()
**
**	 
**
**  **avb
**
****************************/
	
void refresh()
{
	actrTransactionManager->queueForGraphicsFlush();
	actrTransactionManager->flushGraphics();
	acedUpdateDisplay();
}

/******************************************************************
**  void select()
**  
**	  
**
**  **avb
**
****************************/
int select()
{
	AcToRtf ar;
	
	AcDbEntity* pEnt=NULL;
	AcDbMText* pMText;
	AcDbObjectId  eId;
	AcDbObjectId styleId;
	AcDbDimension* pDim;
	ads_point pt;
	int result;
	mglstring.dim=FALSE;
	result=acedEntSel("\n : ",mglstring.name , pt);
	if (result == RTCAN) 
		return 1;
	if (result == RTERROR )
	{
		int val;
		getSysVar("ERRNO", val);
		if (val==7)
			return 2;
		else 
			return 1;
	}	if (!mglstring.name)
		return 1;
	
	acdbGetObjectId(eId, mglstring.name);
    acdbOpenObject(pEnt, eId, AcDb::kForWrite);
	//const char *pCname = pEnt->isA()->name();
	/*if (result=strcmp(pCname,"AcDbAttributeDefinition")==0)
	{
		pEnt->close(); 
		const char* csh="\n  ...";
		acutPrintf(csh);
		acutPrintf(pCname);
		return 2;
	}*/

	if (pEnt->isKindOf(AcDbMText::desc()))
	{
	pMText = AcDbMText::cast(pEnt);
	mglstring.height=pMText->textHeight();	
	mglstring.string=ar.getText(pMText->contents());
	mglstring.alignmentPoint=pMText->alignmentPoint();
	mglstring.horizontalMode=pMText->horizontalMode();
	mglstring.lineSpace=pMText->lineSpacingFactor();
	mglstring.lspstyle=pMText->lineSpacingStyle();
	mglstring.position=pMText->location();
	styleId=pMText->textStyle();
	getTextStyleName(mglstring.textStyle,styleId);
	mglstring.rotation=pMText->rotation() ;
	mglstring.edit=TRUE;
	mglstring.widthFactor=pMText->width();
	pEnt->close();

	}
	else if (pEnt->isKindOf(AcDbDimension::desc()))
	{
		mglstring.dim=TRUE;
		pDim = AcDbDimension::cast(pEnt);
		mglstring.string=ar.getText((const char*)pDim->dimensionText()) ;
		pEnt->close();
		mglstring.edit=TRUE;

	}

	else 
	{
		const char *pCname = pEnt->isA()->name();
		pEnt->close(); 
		const char* csh="\n  ...";
		acutPrintf(csh);
		acutPrintf(pCname);
		return 2;
	}

	
	return 0;
}

